from flask import Flask, render_template, redirect, url_for, g, request # Added request import
import os
import logging
from config import TIME_ZONES
from models.base import initialize_database
from app.utils.logging_config import setup_logging
from app.utils.filters import relative_time
from app.utils.scheduler import init_scheduler
from app.routes import auth, dashboard, main, messages, settings, author_api, message_api
from models.user_session import UserSession
from models.user import User

logger = logging.getLogger(__name__)

def create_app():
    # Explicitly set the template folder relative to the app directory
    app = Flask(__name__, template_folder='../templates', static_folder='../static', static_url_path='/static')

    # Configuration
    app.secret_key = os.getenv('SECRET_KEY', 'dev-key-for-development-only')
    app.config['IMAGE_SAVE_PATH'] = os.path.join(app.static_folder, 'images')
    app.config['VIDEOS_SAVE_PATH'] = os.path.join(app.static_folder, 'videos')
    app.config['LOG_DIR'] = 'logs'
    app.config['BASE_DIR'] = os.path.abspath(os.path.dirname(__file__))

    # Initialize database
    initialize_database()

    # Logging setup
    setup_logging()

    # Jinja2 filter
    app.jinja_env.filters['relative_time'] = relative_time

    # Register blueprints
    app.register_blueprint(auth.bp)
    app.register_blueprint(dashboard.bp)
    app.register_blueprint(main.bp)
    app.register_blueprint(messages.bp)
    app.register_blueprint(settings.bp)
    app.register_blueprint(author_api.bp)
    app.register_blueprint(message_api.bp)

    # before_request handler
    @app.before_request
    def load_logged_in_user():
        g.user = None
        # Skip auth check for static files and auth-related endpoints
        if request.path.startswith('/static') or \
           request.path in ['/auth/login', '/auth/register', '/api/auth/login', '/api/auth/register']:
            return

        auth_token = request.cookies.get('session_token')
        if auth_token:
            user_id = UserSession.validate_session(auth_token)
            if user_id:
                g.user = User.get_by_id(user_id)

    # Error handlers
    @app.errorhandler(401)
    def unauthorized(error):
        return redirect(url_for('auth.login_page'))

    @app.errorhandler(403)
    def forbidden(error):
        return render_template('403.html'), 403


    return app
